#ifndef MACH_MSG_SERVER_IDLING_H
#define MACH_MSG_SERVER_IDLING_H

#include <mach/mach.h>

/* 
 * mach_msg_server_idling
 *
 * Works similar to mach_msg_server, but it calls an idle function periodically.
 * When a request is not received from the client for timeout seconds, the idle
 * function is called. The idle function can return a non-zero value to indicate
 * that the server should continue running, or a zero to indicate the server should
 * be terminated. When the idle function retuns 0, mach_msg_server_idling exits
 * the server loop and returns MACH_RCV_TIMED_OUT to the caller.
 */

#if __cplusplus
extern "C" {
#endif

mach_msg_return_t
mach_msg_server_idling(
    const char *inServiceName,
    boolean_t (*demux)(mach_msg_header_t *, mach_msg_header_t *),
    boolean_t (*idle)(void),
    mach_msg_timeout_t timeout,
    mach_msg_size_t max_size,
    mach_port_t rcv_name,
    mach_msg_options_t options);

#if __cplusplus
};
#endif

#endif /* MACH_MSG_SERVER_IDLING_H */
